/*
  Copyright (C) 2000-2005 SKYRIX Software AG

  This file is part of SOPE.

  SOPE is free software; you can redistribute it and/or modify it under
  the terms of the GNU Lesser General Public License as published by the
  Free Software Foundation; either version 2, or (at your option) any
  later version.

  SOPE is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
  License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with SOPE; see the file COPYING.  If not, write to the
  Free Software Foundation, 59 Temple Place - Suite 330, Boston, MA
  02111-1307, USA.
*/

#ifndef __NGMime_NGMimeJoinedData_H__
#define __NGMime_NGMimeJoinedData_H__

#import <Foundation/NSData.h>

/*
  NGMimeJoinedData
  
  TODO: explain.

  Note: -copyWithZone: returns a _retained_ copy. Bug or feature?
*/

@class NSMutableArray;

@interface NGMimeJoinedData : NSObject
{
  NSMutableArray *joinedDataObjects;
}

- (void)appendData:(NSData *)_data;

- (void)appendBytes:(const void *)_bytes length:(unsigned int)_length;

- (BOOL)writeToFile:(NSString*)_path atomically:(BOOL)_useAuxiliaryFile;

@end /* NGMimeJoinedData */

#endif /* __NGMime_NGMimeJoinedData_H__ */
