/*
  Copyright (C) 2000-2005 SKYRIX Software AG

  This file is part of SOPE.

  SOPE is free software; you can redistribute it and/or modify it under
  the terms of the GNU Lesser General Public License as published by the
  Free Software Foundation; either version 2, or (at your option) any
  later version.

  SOPE is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
  License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with SOPE; see the file COPYING.  If not, write to the
  Free Software Foundation, 59 Temple Place - Suite 330, Boston, MA
  02111-1307, USA.
*/

#ifndef __WEExtensions_WETreeContextKeys_H__
#define __WEExtensions_WETreeContextKeys_H__

#import <Foundation/NSString.h>

extern NSString *WETreeView_HEADER_MODE;
extern NSString *WETreeView_ZOOM_ACTION_ID;

extern NSString *WETreeView_TreeElement;
extern NSString *WETreeView_RenderNoTable;

extern NSString *WETreeView_IconWidth;
extern NSString *WETreeView_Plus;
extern NSString *WETreeView_Minus;
extern NSString *WETreeView_Leaf;
extern NSString *WETreeView_Line;
extern NSString *WETreeView_Junction;
extern NSString *WETreeView_Corner;
extern NSString *WETreeView_CornerPlus;
extern NSString *WETreeView_CornerMinus;
extern NSString *WETreeView_CornerLeaf;
extern NSString *WETreeView_Space;

#endif /* __WEExtensions_WETreeContextKeys_H__ */
